var request_1 = require('./request');
var common_1 = require('./common');
var index_1 = require('./plugins/index');
request_1.default.prototype._open = open;
request_1.default.prototype._abort = abort;
request_1.default.prototype._use = index_1.defaults;
function getXHR() {
    if (XMLHttpRequest) {
        return new XMLHttpRequest();
    }
    try {
        return new ActiveXObject('Microsoft.XMLHTTP');
    }
    catch (e) { }
    try {
        return new ActiveXObject('Msxml2.XMLHTTP.6.0');
    }
    catch (e) { }
    try {
        return new ActiveXObject('Msxml2.XMLHTTP.3.0');
    }
    catch (e) { }
    try {
        return new ActiveXObject('Msxml2.XMLHTTP');
    }
    catch (e) { }
    throw new TypeError('XMLHttpRequest is not available');
}
function parseRawHeaders(xhr) {
    var headers = {};
    var lines = xhr.getAllResponseHeaders().split(/\r?\n/);
    lines.pop();
    lines.forEach(function (header) {
        var indexOf = header.indexOf(':');
        var name = header.substr(0, indexOf);
        var value = header.substr(indexOf + 1).trim();
        if (!headers.hasOwnProperty(name)) {
            headers[name] = value;
        }
        else if (typeof headers[name] === 'string') {
            headers[name] = [headers[name], value];
        }
        else {
            headers[name].push(value);
        }
    });
    return headers;
}
function open(request) {
    return new Promise(function (resolve, reject) {
        var url = request.fullUrl();
        var method = request.method;
        if (window.location.protocol === 'https:' && /^http\:/.test(url)) {
            return reject(request.error("The request to \"" + url + "\" was blocked", 'EBLOCKED'));
        }
        var xhr = request.raw = getXHR();
        xhr.onload = function () {
            return resolve({
                status: xhr.status === 1223 ? 204 : xhr.status,
                headers: parseRawHeaders(xhr),
                body: xhr.responseText,
                url: xhr.responseURL
            });
        };
        xhr.onabort = function () {
            return reject(request.error('Request aborted', 'EABORT'));
        };
        xhr.onerror = function () {
            return reject(request.error("Unable to connect to \"" + request.fullUrl() + "\"", 'EUNAVAILABLE'));
        };
        xhr.onprogress = function (e) {
            if (e.lengthComputable) {
                request.downloadLength = e.total;
            }
            request.downloadedBytes = e.loaded;
        };
        if (method === 'GET' || method === 'HEAD' || !xhr.upload) {
            request.uploadLength = 0;
            request.uploadedBytes = 0;
        }
        else {
            xhr.upload.onprogress = function (e) {
                if (e.lengthComputable) {
                    request.uploadLength = e.total;
                }
                request.uploadedBytes = e.loaded;
            };
        }
        try {
            xhr.open(method, url);
        }
        catch (e) {
            return reject(request.error("Refused to connect to \"" + url + "\"", 'ECSP', e));
        }
        if (request.options.withCredentials) {
            xhr.withCredentials = true;
        }
        Object.keys(request.headers).forEach(function (header) {
            xhr.setRequestHeader(request.name(header), request.get(header));
        });
        xhr.send(request.body);
    });
}
function abort(request) {
    request.raw.abort();
}
module.exports = common_1.defaults({});
//# sourceMappingURL=browser.js.map